@@ -50,6 +50,8 @@
 
 #include <iba/ib_types.h>
 #include <complib/cl_debug.h>
+#include <opensm/osm_file_ids.h>
+#define FILE_ID OSM_FILE_CONGESTION_CONTROL_C
 #include <opensm/osm_subnet.h>
 #include <opensm/osm_opensm.h>
 #include <opensm/osm_log.h>
@@ -521,16 +523,20 @@ static void cc_poller_send(osm_congestion_control_t *p_cc,
 {
 	osm_subn_opt_t *p_opt = &p_cc->subn->opt;
 	ib_api_status_t status;
+	cl_status_t sts;
 	osm_madw_context_t mad_context = p_madw->context;
 
 	status = osm_vendor_send(p_cc->bind_handle, p_madw, TRUE);
 	if (status == IB_SUCCESS) {
 		cl_atomic_inc(&p_cc->outstanding_mads_on_wire);
 		while (p_cc->outstanding_mads_on_wire >
-		       (int32_t)p_opt->cc_max_outstanding_mads)
-			cl_event_wait_on(&p_cc->sig_mads_on_wire_continue,
-					 EVENT_NO_TIMEOUT,
-					 TRUE);
+		       (int32_t)p_opt->cc_max_outstanding_mads) {
+wait:
+			sts = cl_event_wait_on(&p_cc->sig_mads_on_wire_continue,
+					       EVENT_NO_TIMEOUT, TRUE);
+			if (sts != CL_SUCCESS)
+				goto wait;
+		}
 	} else
 		OSM_LOG(p_cc->log, OSM_LOG_ERROR, "ERR C104: "
 			"send failed to node 0x%" PRIx64 "port %u\n",
@@ -667,7 +673,7 @@ static void cc_mad_send_err_callback(void *bind_context,
 
 	p_port = osm_get_port_by_guid(p_cc->subn, port_guid);
 	if (!p_port) {
-		OSM_LOG(p_cc->log, OSM_LOG_ERROR, "ERR C106A: "
+		OSM_LOG(p_cc->log, OSM_LOG_ERROR, "ERR C10B: "
 			"Port guid not in table 0x%" PRIx64 "\n",
 			port_guid);
 		cl_plock_release(&p_osm->lock);
