@@ -3011,6 +3011,8 @@ const char *osm_get_manufacturer_str(IN uint64_t guid_ho)
 		return intel_str;
 	case OSM_VENDOR_ID_MELLANOX:
 	case OSM_VENDOR_ID_MELLANOX2:
+	case OSM_VENDOR_ID_MELLANOX3:
+	case OSM_VENDOR_ID_MELLANOX4:
 		return mellanox_str;
 	case OSM_VENDOR_ID_REDSWITCH:
 		return redswitch_str;
@@ -3125,7 +3127,7 @@ static const char *lsea_str_fixed_width[] = {
 const char *osm_get_lsa_str(IN uint8_t lsa, IN uint8_t lsea, IN uint8_t state,
 			    IN uint8_t fdr10)
 {
-	if (lsa > IB_LINK_SPEED_ACTIVE_10)
+	if (lsa > IB_LINK_SPEED_ACTIVE_10 || state == IB_LINK_DOWN)
 		return lsa_str_fixed_width[3];
 	if (lsea == IB_LINK_SPEED_EXT_ACTIVE_NONE) {
 		if (fdr10)
