@@ -2,7 +2,8 @@
  * Copyright (c) 2007 The Regents of the University of California.
  * Copyright (c) 2007-2009 Voltaire, Inc. All rights reserved.
  * Copyright (c) 2009,2010 HNR Consulting. All rights reserved.
- * Copyright (c) 2013 Lawrence Livermore National Security. All rights * reserved.
+ * Copyright (c) 2013 Lawrence Livermore National Security. All rights reserved.
+ * Copyright (c) 2011-2014 Mellanox Technologies LTD. All rights reserved.
  *
  * This software is available to you under a choice of one of two
  * licenses.  You may choose to be licensed under the terms of the GNU
@@ -162,6 +163,17 @@ static void remove_marked_nodes(osm_perfmgr_t * pm)
 static inline void decrement_outstanding_queries(osm_perfmgr_t * pm)
 {
 	cl_atomic_dec(&pm->outstanding_queries);
+
+	if (!pm->outstanding_queries) {
+		cl_spinlock_acquire(&pm->lock);
+		if (pm->sweep_state == PERFMGR_SWEEP_POST_PROCESSING) {
+			pm->sweep_state = PERFMGR_SWEEP_SLEEP;
+			OSM_LOG(pm->log, OSM_LOG_INFO,
+				"PM sweep state exiting Post Processing\n");
+		}
+		cl_spinlock_release(&pm->lock);
+	}
+
 	cl_event_signal(&pm->sig_query);
 }
 
@@ -429,6 +441,7 @@ static osm_madw_t *perfmgr_build_mad(osm_perfmgr_t * perfmgr,
 static ib_api_status_t perfmgr_send_mad(osm_perfmgr_t *perfmgr,
 					osm_madw_t * const p_madw)
 {
+	cl_status_t sts;
 	ib_api_status_t status = osm_vendor_send(perfmgr->bind_handle, p_madw,
 						 TRUE);
 	if (status == IB_SUCCESS) {
@@ -437,20 +450,23 @@ static ib_api_status_t perfmgr_send_mad(osm_perfmgr_t *perfmgr,
 		while (perfmgr->outstanding_queries >
 		       (int32_t)perfmgr->max_outstanding_queries) {
 			cl_spinlock_acquire(&perfmgr->lock);
-			perfmgr->sweep_state = PERFMGR_SWEEP_SUSPENDED;
+			if (perfmgr->sweep_state == PERFMGR_SWEEP_SLEEP) {
+				perfmgr->sweep_state = PERFMGR_SWEEP_POST_PROCESSING;
+				OSM_LOG(perfmgr->log, OSM_LOG_INFO,
+					"PM sweep state going into Post Processing\n");
+			} else if (perfmgr->sweep_state == PERFMGR_SWEEP_ACTIVE)
+				perfmgr->sweep_state = PERFMGR_SWEEP_SUSPENDED;
 			cl_spinlock_release(&perfmgr->lock);
-			cl_event_wait_on(&perfmgr->sig_query, EVENT_NO_TIMEOUT,
-					 TRUE);
+wait:
+			sts = cl_event_wait_on(&perfmgr->sig_query,
+					       EVENT_NO_TIMEOUT, TRUE);
+			if (sts != CL_SUCCESS)
+				goto wait;
 
 			cl_spinlock_acquire(&perfmgr->lock);
-			if (perfmgr->sweep_state == PERFMGR_SWEEP_SUSPENDED) {
+			if (perfmgr->sweep_state == PERFMGR_SWEEP_SUSPENDED)
 				perfmgr->sweep_state = PERFMGR_SWEEP_ACTIVE;
-				cl_spinlock_release(&perfmgr->lock);
-			} else {
-				cl_spinlock_release(&perfmgr->lock);
-				OSM_LOG(perfmgr->log, OSM_LOG_ERROR, "ERR 54FF: "
-					"PM was NOT in Suspended state???\n");
-			}
+			cl_spinlock_release(&perfmgr->lock);
 		}
 	}
 	return (status);
@@ -465,6 +481,7 @@ static ib_api_status_t perfmgr_send_pc_mad(osm_perfmgr_t * perfmgr,
 					   ib_net32_t dest_qp, uint16_t pkey_ix,
 					   uint8_t port, uint8_t mad_method,
 					   uint16_t counter_select,
+					   uint8_t counter_select2,
 					   osm_madw_context_t * p_context,
 					   uint8_t sl)
 {
@@ -486,6 +503,7 @@ static ib_api_status_t perfmgr_send_pc_mad(osm_perfmgr_t * perfmgr,
 	memset(port_counter, 0, sizeof(*port_counter));
 	port_counter->port_select = port;
 	port_counter->counter_select = cl_hton16(counter_select);
+	port_counter->counter_select2 = counter_select2;
 
 	status = perfmgr_send_mad(perfmgr, p_madw);
 
@@ -593,6 +611,16 @@ static inline boolean_t pce_supported(monitored_node_t *mon_node, uint8_t port)
 		|| mon_port->cap_mask & IB_PM_EXT_WIDTH_NOIETF_SUP));
 }
 
+/**********************************************************************
+ * return if CapMask.PortCountersXmitWaitSupported is set
+ **********************************************************************/
+static inline boolean_t xmit_wait_supported(monitored_node_t *mon_node, uint8_t port)
+{
+	monitored_port_t *mon_port = &(mon_node->port[port]);
+	return (mon_port->cpi_valid
+		&& (mon_port->cap_mask & IB_PM_PC_XMIT_WAIT_SUP));
+}
+
 /**********************************************************************
  * return if "full" PortCountersExtended (IETF) is indicated
  **********************************************************************/
@@ -728,12 +756,14 @@ static void perfmgr_query_counters(cl_map_item_t * p_map_item, void *context)
 			gettimeofday(&mad_context.perfmgr_context.query_start, NULL);
 #endif
 			OSM_LOG(pm->log, OSM_LOG_VERBOSE, "Getting stats for node 0x%"
-				PRIx64 " port %d (lid %u) (%s)\n", node_guid, port,
-				cl_ntoh16(lid), node->print_desc);
+				PRIx64 " port %d (lid %u) (%s)\n",
+				node_guid, port, cl_ntoh16(lid),
+				node->print_desc);
 			status = perfmgr_send_pc_mad(pm, lid, remote_qp,
 						     mon_node->port[port].pkey_ix,
 						     port, IB_MAD_METHOD_GET,
 						     0xffff,
+						     1,
 						     &mad_context,
 						     0); /* FIXME SL != 0 */
 			if (status != IB_SUCCESS)
@@ -1001,8 +1031,12 @@ void osm_perfmgr_process(osm_perfmgr_t * pm)
 
 	cl_spinlock_acquire(&pm->lock);
 	if (pm->sweep_state == PERFMGR_SWEEP_ACTIVE ||
-	    pm->sweep_state == PERFMGR_SWEEP_SUSPENDED) {
+	    pm->sweep_state == PERFMGR_SWEEP_SUSPENDED ||
+	    pm->sweep_state == PERFMGR_SWEEP_POST_PROCESSING) {
 		cl_spinlock_release(&pm->lock);
+		OSM_LOG(pm->log, OSM_LOG_INFO,
+			"PM sweep state %d, skipping sweep\n",
+			pm->sweep_state);
 		return;
 	}
 
@@ -1052,10 +1086,6 @@ void osm_perfmgr_process(osm_perfmgr_t * pm)
 	remove_marked_nodes(pm);
 
 #ifdef ENABLE_OSM_PERF_MGR_PROFILE
-	/* spin on outstanding queries */
-	while (pm->outstanding_queries > 0)
-		cl_event_wait_on(&pm->sig_sweep, 1000, TRUE);
-
 	gettimeofday(&after, NULL);
 	diff_time(&before, &after, &after);
 	osm_log_v2(pm->log, OSM_LOG_INFO, FILE_ID,
@@ -1139,6 +1169,7 @@ static void perfmgr_check_oob_clear(osm_perfmgr_t * pm,
 		"LI:   %"PRIu64" ?< %"PRIu64"\n"
 		"BO:   %"PRIu64" ?< %"PRIu64"\n"
 		"VL15: %"PRIu64" ?< %"PRIu64"\n"
+		"XW:   %"PRIu64" ?< %"PRIu64"\n"
 		,
 		mon_node->name, mon_node->guid, port,
 		cr->symbol_err_cnt, prev_err.symbol_err_cnt,
@@ -1152,7 +1183,8 @@ static void perfmgr_check_oob_clear(osm_perfmgr_t * pm,
 		cr->rcv_constraint_err, prev_err.rcv_constraint_err,
 		cr->link_integrity, prev_err.link_integrity,
 		cr->buffer_overrun, prev_err.buffer_overrun,
-		cr->vl15_dropped, prev_err.vl15_dropped);
+		cr->vl15_dropped, prev_err.vl15_dropped,
+		cr->xmit_wait, prev_err.xmit_wait);
 
 	if (cr->symbol_err_cnt < prev_err.symbol_err_cnt ||
 	    cr->link_err_recover < prev_err.link_err_recover ||
@@ -1165,7 +1197,8 @@ static void perfmgr_check_oob_clear(osm_perfmgr_t * pm,
 	    cr->rcv_constraint_err < prev_err.rcv_constraint_err ||
 	    cr->link_integrity < prev_err.link_integrity ||
 	    cr->buffer_overrun < prev_err.buffer_overrun ||
-	    cr->vl15_dropped < prev_err.vl15_dropped) {
+	    cr->vl15_dropped < prev_err.vl15_dropped ||
+	    cr->xmit_wait < prev_err.xmit_wait) {
 		OSM_LOG(pm->log, OSM_LOG_ERROR, "PerfMgr: ERR 540A: "
 			"Detected an out of band error clear "
 			"on %s (0x%" PRIx64 ") port %u\n",
@@ -1209,12 +1242,14 @@ static int counter_overflow_64(ib_net64_t val)
  **********************************************************************/
 static void perfmgr_check_overflow(osm_perfmgr_t * pm,
 				   monitored_node_t * mon_node, int16_t pkey_ix,
-				   uint8_t port, ib_port_counters_t * pc)
+				   uint8_t port, ib_port_counters_t * pc,
+				   boolean_t xmit_wait_sup)
 {
 	osm_madw_context_t mad_context;
 	ib_api_status_t status;
 	ib_net32_t remote_qp;
 	uint16_t counter_select;
+	uint8_t counter_select2;
 
 	OSM_LOG_ENTER(pm->log);
 
@@ -1230,6 +1265,7 @@ static void perfmgr_check_overflow(osm_perfmgr_t * pm,
 	    counter_overflow_4(PC_LINK_INT(pc->link_int_buffer_overrun)) ||
 	    counter_overflow_4(PC_BUF_OVERRUN(pc->link_int_buffer_overrun)) ||
 	    counter_overflow_16(pc->vl15_dropped) ||
+	    (xmit_wait_sup && counter_overflow_32(pc->xmit_wait)) ||
 	    (!pce_supported(mon_node, port) &&
 	    (counter_overflow_32(pc->xmit_data) ||
 	     counter_overflow_32(pc->rcv_data) ||
@@ -1275,9 +1311,15 @@ static void perfmgr_check_overflow(osm_perfmgr_t * pm,
 		else
 			counter_select = 0xffff;
 
+		if (xmit_wait_sup)
+			counter_select2 = 1;
+		else
+			counter_select2 = 0;
+
 		status = perfmgr_send_pc_mad(pm, lid, remote_qp, pkey_ix,
 					     port, IB_MAD_METHOD_SET,
 					     counter_select,
+					     counter_select2,
 					     &mad_context,
 					     0); /* FIXME SL != 0 */
 		if (status != IB_SUCCESS)
@@ -1377,6 +1419,7 @@ static void perfmgr_log_errors(osm_perfmgr_t * pm,
 	perfmgr_db_err_reading_t prev_read;
 	perfmgr_db_err_t err =
 	    perfmgr_db_get_prev_err(pm->db, mon_node->guid, port, &prev_read);
+	uint64_t cur, prev;
 
 	if (err != PERFMGR_EVENT_DB_SUCCESS) {
 		OSM_LOG(pm->log, OSM_LOG_VERBOSE, "Failed to find previous "
@@ -1406,6 +1449,15 @@ static void perfmgr_log_errors(osm_perfmgr_t * pm,
 	LOG_ERR_CNT("LocalLinkIntegrityErrors",     "543A", link_integrity);
 	LOG_ERR_CNT("ExcessiveBufferOverrunErrors", "543B", buffer_overrun);
 	LOG_ERR_CNT("VL15Dropped",                  "543C", vl15_dropped);
+
+	cur = reading->xmit_wait;
+	prev = prev_read.xmit_wait;
+	if (pm->xmit_wait_log && cur > prev &&
+	    (cur - prev) >= pm->xmit_wait_threshold) {
+		OSM_LOG(pm->log, OSM_LOG_ERROR, "ERR 543D: XmitWait : %" PRIu64
+			" : node \"%s\" (NodeGUID: 0x%" PRIx64 ") : port %u\n",
+			cur - prev, mon_node->name, mon_node->guid, port);
+	}
 }
 
 static int16_t validate_redir_pkey(osm_perfmgr_t *pm, ib_net16_t pkey)
@@ -1555,13 +1607,22 @@ static boolean_t handle_redirect(osm_perfmgr_t *pm,
 						0); /* FIXME SL != 0 */
 	} else {
 		/* reissue the original query to the redirected location */
+		uint8_t counter_select2;
+
+		if (xmit_wait_supported(p_mon_node, port))
+			counter_select2 = 1;
+		else
+			counter_select2 = 0;
+
 		mad_method = mad_context->perfmgr_context.mad_method;
 		if (mad_context->perfmgr_context.mad_attr_id
 		    == IB_MAD_ATTR_PORT_CNTRS) {
-			status = perfmgr_send_pc_mad(pm, cpi->redir_lid, cpi->redir_qp,
+			status = perfmgr_send_pc_mad(pm, cpi->redir_lid,
+						     cpi->redir_qp,
 						     pkey_ix, port,
 						     mad_method,
 						     0xffff,
+						     counter_select2,
 						     mad_context,
 						     0); /* FIXME SL != 0 */
 		} else {
@@ -1714,7 +1775,7 @@ static void pc_recv_process(void *context, void *data)
 		if (pm->query_cpi && cpi_valid) {
 			cl_plock_acquire(&pm->osm->lock);
 			if (p_mon_node->node_type == IB_NODE_TYPE_SWITCH) {
-				int i = 0;
+				int i;
 				for (i = p_mon_node->esp0 ? 0 : 1;
 				     i < p_mon_node->num_ports;
 				     i++) {
@@ -1760,11 +1821,12 @@ static void pc_recv_process(void *context, void *data)
 					   port, ext_wire_read);
 	} else {
 		boolean_t pce_sup = pce_supported(p_mon_node, port);
+		boolean_t xmit_wait_sup = xmit_wait_supported(p_mon_node, port);
 		ib_port_counters_t *wire_read =
 				(ib_port_counters_t *)
 				&osm_madw_get_perfmgt_mad_ptr(p_madw)->data;
 
-		perfmgr_db_fill_err_read(wire_read, &err_reading);
+		perfmgr_db_fill_err_read(wire_read, &err_reading, xmit_wait_sup);
 		if (!pce_sup)
 			perfmgr_db_fill_data_cnt_read_pc(wire_read, &data_reading);
 
@@ -1791,7 +1853,7 @@ static void pc_recv_process(void *context, void *data)
 		}
 
 		perfmgr_check_overflow(pm, p_mon_node, p_mon_node->port[port].pkey_ix,
-				       port, wire_read);
+				       port, wire_read, xmit_wait_sup);
 
 	}
 
@@ -1825,8 +1887,6 @@ ib_api_status_t osm_perfmgr_init(osm_perfmgr_t * pm, osm_opensm_t * osm,
 
 	memset(pm, 0, sizeof(*pm));
 
-	cl_event_construct(&pm->sig_sweep);
-	cl_event_init(&pm->sig_sweep, FALSE);
 	pm->subn = &osm->subn;
 	pm->sm = &osm->sm;
 	pm->log = &osm->log;
@@ -1868,6 +1928,8 @@ ib_api_status_t osm_perfmgr_init(osm_perfmgr_t * pm, osm_opensm_t * osm,
 
 	pm->rm_nodes = p_opt->perfmgr_rm_nodes;
 	pm->query_cpi = p_opt->perfmgr_query_cpi;
+	pm->xmit_wait_log = p_opt->perfmgr_xmit_wait_log;
+	pm->xmit_wait_threshold = p_opt->perfmgr_xmit_wait_threshold;
 	status = IB_SUCCESS;
 Exit:
 	OSM_LOG_EXIT(pm->log);
