@@ -275,12 +275,58 @@ static void opts_setup_sm_priority(osm_subn_t *p_subn, void *p_val)
 	osm_set_sm_priority(p_sm, sm_priority);
 }
 
+static int opts_strtoul(uint32_t *val, IN char *p_val_str,
+			IN char *p_key, uint32_t max_value)
+{
+	char *endptr;
+	unsigned long int tmp_val;
+
+	errno = 0;
+	tmp_val = strtoul(p_val_str, &endptr, 0);
+	*val = tmp_val;
+	if (*p_val_str == '\0' || *endptr != '\0') {
+		log_report("-E- Parsing error in field %s, expected "
+			   "numeric input received: %s\n", p_key, p_val_str);
+		return -1;
+	}
+	if (tmp_val > max_value ||
+	    ((tmp_val == ULONG_MAX) && errno == ERANGE)) {
+		log_report("-E- Parsing error in field %s, value out of range\n", p_key);
+		return -1;
+	}
+	return 0;
+}
+
+static int opts_strtoull(uint64_t *val, IN char *p_val_str,
+			 IN char *p_key, uint64_t max_value)
+{
+	char *endptr;
+	unsigned long long int tmp_val;
+
+	errno = 0;
+	tmp_val = strtoull(p_val_str, &endptr, 0);
+	*val = tmp_val;
+	if (*p_val_str == '\0' || *endptr != '\0') {
+		log_report("-E- Parsing error in field %s, expected "
+			   "numeric input received: %s\n", p_key, p_val_str);
+		return -1;
+	}
+	if (tmp_val > max_value || (tmp_val == ULLONG_MAX && errno == ERANGE)) {
+		log_report("-E- Parsing error in field %s, value out of range", p_key);
+		return -1;
+	}
+	return 0;
+}
+
 static void opts_parse_net64(IN osm_subn_t *p_subn, IN char *p_key,
 			     IN char *p_val_str, void *p_v1, void *p_v2,
 			     void (*pfn)(osm_subn_t *, void *))
 {
 	uint64_t *p_val1 = p_v1, *p_val2 = p_v2;
-	uint64_t val = strtoull(p_val_str, NULL, 0);
+	uint64_t val;
+
+	if (opts_strtoull(&val, p_val_str, p_key, UINT64_MAX))
+		return;
 
 	if (cl_hton64(val) != *p_val1) {
 		log_config_value(p_key, "0x%016" PRIx64, val);
@@ -295,7 +341,10 @@ static void opts_parse_uint32(IN osm_subn_t *p_subn, IN char *p_key,
 			      void (*pfn)(osm_subn_t *, void *))
 {
 	uint32_t *p_val1 = p_v1, *p_val2 = p_v2;
-	uint32_t val = strtoul(p_val_str, NULL, 0);
+	uint32_t val;
+
+	if (opts_strtoul(&val, p_val_str, p_key, UINT32_MAX))
+		return;
 
 	if (val != *p_val1) {
 		log_config_value(p_key, "%u", val);
@@ -310,7 +359,10 @@ static void opts_parse_net32(IN osm_subn_t *p_subn, IN char *p_key,
 			     void (*pfn)(osm_subn_t *, void *))
 {
 	uint32_t *p_val1 = p_v1, *p_val2 = p_v2;
-	uint32_t val = strtoul(p_val_str, NULL, 0);
+	uint32_t val;
+
+	if (opts_strtoul(&val, p_val_str, p_key, UINT32_MAX))
+		return;
 
 	if (cl_hton32(val) != *p_val1) {
 		log_config_value(p_key, "%u", val);
@@ -320,7 +372,6 @@ static void opts_parse_net32(IN osm_subn_t *p_subn, IN char *p_key,
 	}
 }
 
-
 static void opts_parse_int32(IN osm_subn_t *p_subn, IN char *p_key,
 			     IN char *p_val_str, void *p_v1, void *p_v2,
 			     void (*pfn)(osm_subn_t *, void *))
@@ -341,8 +392,12 @@ static void opts_parse_uint16(IN osm_subn_t *p_subn, IN char *p_key,
 			      void (*pfn)(osm_subn_t *, void *))
 {
 	uint16_t *p_val1 = p_v1, *p_val2 = p_v2;
-	uint16_t val = (uint16_t) strtoul(p_val_str, NULL, 0);
+	uint32_t tmp_val;
 
+	if (opts_strtoul(&tmp_val, p_val_str, p_key, UINT16_MAX))
+		return;
+
+	uint16_t val = (uint16_t) tmp_val;
 	if (val != *p_val1) {
 		log_config_value(p_key, "%u", val);
 		if (pfn)
@@ -356,8 +411,12 @@ static void opts_parse_net16(IN osm_subn_t *p_subn, IN char *p_key,
 			     void (*pfn)(osm_subn_t *, void *))
 {
 	uint16_t *p_val1 = p_v1, *p_val2 = p_v2;
-	uint16_t val = strtoul(p_val_str, NULL, 0);
+	uint32_t tmp_val;
 
+	if (opts_strtoul(&tmp_val, p_val_str, p_key, UINT16_MAX))
+		return;
+
+	uint16_t val = (uint16_t) tmp_val;
 	if (cl_hton16(val) != *p_val1) {
 		log_config_value(p_key, "0x%04x", val);
 		if (pfn)
@@ -371,8 +430,12 @@ static void opts_parse_uint8(IN osm_subn_t *p_subn, IN char *p_key,
 			     void (*pfn)(osm_subn_t *, void *))
 {
 	uint8_t *p_val1 = p_v1, *p_val2 = p_v2;
-	uint8_t val = strtoul(p_val_str, NULL, 0);
+	uint32_t tmp_val;
 
+	if (opts_strtoul(&tmp_val, p_val_str, p_key, UINT8_MAX))
+		return;
+
+	uint8_t val = (uint8_t) tmp_val;
 	if (val != *p_val1) {
 		log_config_value(p_key, "%u", val);
 		if (pfn)
@@ -756,6 +819,7 @@ static const opt_rec_t opt_tbl[] = {
 	{ "sm_assigned_guid", OPT_OFFSET(sm_assigned_guid), opts_parse_uint8, NULL, 1 },
 	{ "qos", OPT_OFFSET(qos), opts_parse_boolean, NULL, 1 },
 	{ "qos_policy_file", OPT_OFFSET(qos_policy_file), opts_parse_charp, NULL, 0 },
+	{ "suppress_sl2vl_mad_status_errors", OPT_OFFSET(suppress_sl2vl_mad_status_errors), opts_parse_boolean, NULL, 1 },
 	{ "dump_files_dir", OPT_OFFSET(dump_files_dir), opts_parse_charp, NULL, 0 },
 	{ "lid_matrix_dump_file", OPT_OFFSET(lid_matrix_dump_file), opts_parse_charp, NULL, 0 },
 	{ "lfts_file", OPT_OFFSET(lfts_file), opts_parse_charp, NULL, 0 },
@@ -790,6 +854,8 @@ static const opt_rec_t opt_tbl[] = {
 	{ "perfmgr_rm_nodes", OPT_OFFSET(perfmgr_rm_nodes), opts_parse_boolean, NULL, 0 },
 	{ "perfmgr_log_errors", OPT_OFFSET(perfmgr_log_errors), opts_parse_boolean, NULL, 0 },
 	{ "perfmgr_query_cpi", OPT_OFFSET(perfmgr_query_cpi), opts_parse_boolean, NULL, 0 },
+	{ "perfmgr_xmit_wait_log", OPT_OFFSET(perfmgr_xmit_wait_log), opts_parse_boolean, NULL, 0 },
+	{ "perfmgr_xmit_wait_threshold", OPT_OFFSET(perfmgr_xmit_wait_threshold), opts_parse_uint32, NULL, 0 },
 #endif				/* ENABLE_OSM_PERF_MGR */
 	{ "event_plugin_name", OPT_OFFSET(event_plugin_name), opts_parse_charp, NULL, 0 },
 	{ "event_plugin_options", OPT_OFFSET(event_plugin_options), opts_parse_charp, NULL, 0 },
@@ -893,6 +959,7 @@ static void subn_validate_g2m(osm_subn_t *p_subn)
 					"guid:0x%016" PRIx64 "\n",
 					p_item->guid);
 		}
+		free(p_item);
 	}
 
 Exit:
@@ -959,7 +1026,7 @@ static void subn_validate_neighbor(osm_subn_t *p_subn)
 			valid_entry = FALSE;
 		} else if (osm_db_neighbor_get(p_subn->p_neighbor,
 					       guid, port, &guid, &port) ||
-			guid != p_item->guid || port != p_item->portnum) {
+			   guid != p_item->guid || port != p_item->portnum) {
 			OSM_LOG(&(p_subn->p_osm->log), OSM_LOG_ERROR,
 				"ERR 7518: neighbor does not point "
 				"back at us (guid: 0x%016" PRIx64
@@ -977,6 +1044,7 @@ static void subn_validate_neighbor(osm_subn_t *p_subn)
 					"guid:0x%016" PRIx64 " port:%u\n",
 					p_item->guid, p_item->portnum);
 		}
+		free(p_item);
 	}
 
 Exit:
@@ -1496,7 +1564,9 @@ void osm_subn_set_default_opt(IN osm_subn_opt_t * p_opt)
 	p_opt->event_db_dump_file = NULL; /* use default */
 	p_opt->perfmgr_rm_nodes = TRUE;
 	p_opt->perfmgr_log_errors = TRUE;
-	p_opt->perfmgr_query_cpi = FALSE;
+	p_opt->perfmgr_query_cpi = TRUE;
+	p_opt->perfmgr_xmit_wait_log = FALSE;
+	p_opt->perfmgr_xmit_wait_threshold = OSM_PERFMGR_DEFAULT_XMIT_WAIT_THRESHOLD;
 #endif				/* ENABLE_OSM_PERF_MGR */
 
 	p_opt->event_plugin_name = NULL;
@@ -1517,6 +1587,7 @@ void osm_subn_set_default_opt(IN osm_subn_opt_t * p_opt)
 	p_opt->sm_assigned_guid = 0;
 	p_opt->qos = FALSE;
 	p_opt->qos_policy_file = strdup(OSM_DEFAULT_QOS_POLICY_FILE);
+	p_opt->suppress_sl2vl_mad_status_errors = FALSE;
 	p_opt->accum_log_file = TRUE;
 	p_opt->port_prof_ignore_file = NULL;
 	p_opt->hop_weights_file = NULL;
@@ -2180,7 +2251,7 @@ int osm_subn_rescan_conf_files(IN osm_subn_t * p_subn)
 
 			token_matched = 1;
 
-			if (!r->can_update || strcmp(r->name, p_key))
+			if (!r->can_update)
 				continue;
 
 			p_field1 = (void *)p_opts->file_opts + r->opt_offset;
@@ -2464,8 +2535,8 @@ int osm_subn_output_conf(FILE *out, IN osm_subn_opt_t * p_opts)
 		p_opts->port_shifting ? "TRUE" : "FALSE");
 
 	fprintf(out,
-		"# Assign ports in a random order instead of round-robin.\n"
-		"# If zero disable, otherwise use the value as a random seed\n"
+		"# Assign ports in a random order instead of round-robin\n"
+		"# If zero disable (default), otherwise use the value as a random seed\n"
 		"scatter_ports %d\n\n",
 		p_opts->scatter_ports);
 
@@ -2557,28 +2628,43 @@ int osm_subn_output_conf(FILE *out, IN osm_subn_opt_t * p_opts)
 	fprintf(out,
 		"#\n# Performance Manager Options\n#\n"
 		"# perfmgr enable\n"
+		"# PerfMgr is enabled if TRUE and disabled if FALSE (default FALSE)\n"
 		"perfmgr %s\n\n"
-		"# perfmgr redirection enable\n"
+		"# redirection enable\n"
+		"# Redirection supported if TRUE and not supported if FALSE (default TRUE)\n"
 		"perfmgr_redir %s\n\n"
-		"# sweep time in seconds\n"
+		"# sweep time in seconds (default %u seconds)\n"
 		"perfmgr_sweep_time_s %u\n\n"
-		"# Max outstanding queries\n"
-		"perfmgr_max_outstanding_queries %u\n"
+		"# Max outstanding queries (default %u)\n"
+		"perfmgr_max_outstanding_queries %u\n\n"
+		"# Ignore CAs on sweep (default FALSE)\n"
 		"perfmgr_ignore_cas %s\n\n"
-		"# Remove missing nodes from DB\n"
+		"# Remove missing nodes from DB (default TRUE)\n"
 		"perfmgr_rm_nodes %s\n\n"
-		"# Log error counters to opensm.log\n"
+		"# Log error counters to opensm.log (default TRUE)\n"
 		"perfmgr_log_errors %s\n\n"
-		"# Query PerfMgrGet(ClassPortInfo) for extended capabilities\n"
-		"perfmgr_query_cpi %s\n\n",
+		"# Query PerfMgt Get(ClassPortInfo) for extended capabilities\n"
+		"# Extended capabilities include 64 bit extended counters\n"
+		"# and transmit wait support (default TRUE)\n"
+		"perfmgr_query_cpi %s\n\n"
+		"# Log xmit_wait errors (default FALSE)\n"
+		"perfmgr_xmit_wait_log %s\n\n"
+		"# If logging xmit_wait's; set threshold (default %u)\n"
+		"perfmgr_xmit_wait_threshold %u\n\n"
+		,
 		p_opts->perfmgr ? "TRUE" : "FALSE",
 		p_opts->perfmgr_redir ? "TRUE" : "FALSE",
+		OSM_PERFMGR_DEFAULT_SWEEP_TIME_S,
 		p_opts->perfmgr_sweep_time_s,
+		OSM_PERFMGR_DEFAULT_MAX_OUTSTANDING_QUERIES,
 		p_opts->perfmgr_max_outstanding_queries,
 		p_opts->perfmgr_ignore_cas ? "TRUE" : "FALSE",
 		p_opts->perfmgr_rm_nodes ? "TRUE" : "FALSE",
 		p_opts->perfmgr_log_errors ? "TRUE" : "FALSE",
-		p_opts->perfmgr_query_cpi ? "TRUE" : "FALSE");
+		p_opts->perfmgr_query_cpi ? "TRUE" : "FALSE",
+		p_opts->perfmgr_xmit_wait_log ? "TRUE" : "FALSE",
+		OSM_PERFMGR_DEFAULT_XMIT_WAIT_THRESHOLD,
+		p_opts->perfmgr_xmit_wait_threshold);
 
 	fprintf(out,
 		"#\n# Event DB Options\n#\n"
@@ -2664,8 +2750,11 @@ int osm_subn_output_conf(FILE *out, IN osm_subn_opt_t * p_opts)
 		"# Enable QoS setup\n"
 		"qos %s\n\n"
 		"# QoS policy file to be used\n"
-		"qos_policy_file %s\n\n",
-		p_opts->qos ? "TRUE" : "FALSE", p_opts->qos_policy_file);
+		"qos_policy_file %s\n"
+		"# Supress QoS MAD status errors\n"
+		"suppress_sl2vl_mad_status_errors %s\n\n",
+		p_opts->qos ? "TRUE" : "FALSE", p_opts->qos_policy_file,
+		p_opts->suppress_sl2vl_mad_status_errors ? "TRUE" : "FALSE");
 
 	subn_dump_qos_options(out,
 			      "QoS default options", "qos",
